=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Copyright  2010-2011 Fredo6 - Designed and written December 2010 by Fredo6
#
# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:   SUClockLoader.rb
# Original Date	:   15 sep 2010 - version 1.0
# Description	:   Loader for the Auto Clock script
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module SUClock

#---------------------------------------------------------------------------------------------------------------------------
# Overall Configuration for the Tools	
#---------------------------------------------------------------------------------------------------------------------------

MENU_PERSO = [:T_MenuTools_Fredo6Collection, "Tools", :T_MenuPlugins_Fredo6Collection, "Plugins"]

#---------------------------------------------------------------------------------------------------------------------------
#Constants for TOS  Modules (do not translate here, use Translation Dialog Box instead)	
#---------------------------------------------------------------------------------------------------------------------------

#Menu and Toolbar icons
T6[:MCLK_LAUNCH_Menu] = "Configure Tracking Project"
T6[:MCLK_LAUNCH_Tooltip] = "SUClock: Configuration settings - Show / Hide clock"
MCLK_LAUNCH_Icon = "SUClock"

T6[:MCLK_SCREEN_Menu] = "Move clock within screen"
T6[:MCLK_SCREEN_Tooltip] = "Set the position of the clock within screen"

#--------------------------------------------------------
# Default Parameters
#--------------------------------------------------------

T6[:DEFAULT_SECTION_General] = "Parameters at startup of Sketchup"
T6[:DEFAULT_IdleTime] = "Idle time in minutes"
T6[:DEFAULT_ClockHeight] = "Height of the Clock window (in pixels)"
T6[:DEFAULT_TmpDir] = "Directory for temporary tracker files (useful for sharing on network). Make sure the Directory exists"
T6[:DEFAULT_AlarmThreshold] = "Alarm threshold when tracking target time (in %)"

T6[:DEFAULT_Flag_Export] = "Default Format of Export for Statistics"
T6[:OPT_Export_CSV] = "CSV"
T6[:OPT_Export_Tabbed] = "Tabbed Text"
T6[:OPT_Export_HTML] = "HTML"
OPT_Export = [['CSV', :OPT_Export_CSV], ['TAB', :OPT_Export_Tabbed], ['HTML', :OPT_Export_HTML]]

#--------------------------------------------------------
# Startup routine (called from LibFredo6)
#--------------------------------------------------------
	
#Create all commands with menus and toolbar icons
def SUClock.startup	

	#Top menu
	MYPLUGIN.declare_topmenu nil, MENU_PERSO

	#Declaring command icons
	MYPLUGIN.declare_separator
	MYPLUGIN.declare_command(:MCLK_LAUNCH_) { @tracker.toggle_clock }
	MYPLUGIN.declare_command(:MCLK_SCREEN_) { @tracker.clock_within_screen }
	
	#Declaring the default parameter
	SUClock.default_param
	
	#Startup of the Plugin
	MYPLUGIN.go	
	
	#Installing observer for change of model and reset parameters
	@tracker = Tracker.new
end

#Limits for idle time parameter
def SUClock.idle_limits ; 'I:>=1<=999' ; end
def SUClock.alarm_limits ; 'I:>=50<=100' ; end
	
#Declaring the Default Parameters	
def SUClock.default_param
	dp = MYDEFPARAM
	dp.separator :DEFAULT_SECTION_General
	dp.declare :DEFAULT_IdleTime, 5, SUClock.idle_limits
	dp.declare :DEFAULT_Flag_Export, 'CSV', 'H', T6[OPT_Export]
	dp.declare :DEFAULT_ClockHeight, 50, 'I:>=50<=80'
	dp.declare :DEFAULT_AlarmThreshold, 75, SUClock.alarm_limits
	dp.declare :DEFAULT_TmpDir, "--LOCAL--", 'S'
end
		
end	#End Module SUClock
