=begin
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
# Copyright  2010-2011 Fredo6 - Designed and written December 2010 by Fredo6

# Permission to use this software for any purpose and without fee is hereby granted
# Distribution of this software for commercial purpose is subject to:
#  - the expressed, written consent of the author
#  - the inclusion of the present copyright notice in all copies.

# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#-----------------------------------------------------------------------------
# Name			:   SUClockMain.rb
# Original Date	:   06 Dec 2010 - version 1.0
# Description	:   Main Script for SUClock
#-------------------------------------------------------------------------------------------------------------------------------------------------
#*************************************************************************************************
=end

module SUClock

T6[:TIP_CONFIG] = "Configuration of the Tracker"	
T6[:TIP_RECORD] = "Resume recording"	
T6[:TIP_PAUSE] = "Pause recording"	

T6[:TIP_ShowClock] = "Click to display the Clock"	
T6[:TIP_ChangeSettings] = "Click to change settings"	
T6[:TIP_AttachModel] = "Click to Track the model"	

T6[:TIP_Project] = "Project:"
T6[:TIP_Total] = "Total:"
T6[:TIP_Session] = "Session:"

T6[:DLG_AskSelectFile] = "Selecting SUClock file"
T6[:DLG_AskCreateFile] = "Assigning new SUClock file"
T6[:MSG_InvalidFile] = "INVALID SUClock Project file\n\n%1"
T6[:MSG_SearchAlternate1] = "The model was attached to a Project Tracking File which CANNOT BE FOUND"
T6[:MSG_SearchAlternate2] = "Do you want to search for it?"

T6[:DLG_AdjustClockHeight] = "Height of the clock window (0 to reset to default[50])"

#=============================================================
#--------------------------------------------------------------------------------------------------------------------------
# API for RUBY TOOL REGISTRATION
#--------------------------------------------------------------------------------------------------------------------------
#=============================================================

#Method to be called by a Ruby tool in its Activate method. It associates the Ruby and the Tool ID
#  <ruby> is the name for the Ruby tool
# Example: SUClock.register_ruby 'RoundCorner' if defined?(SUClock)
def SUClock.register_ruby(ruby=nil)
	Traductor::Ruby.register_ruby ruby
end

#=============================================================
#--------------------------------------------------------------------------------------------------------------------------
# Class Tracker: main methods
#--------------------------------------------------------------------------------------------------------------------------
#=============================================================
	
#-----------------------------------------------------------------------------------------------------------------------
# Application Spy via App Observer class
#-----------------------------------------------------------------------------------------------------------------------

class Tracker 

#Observers events
def onNewModel(model) ; model_changing(model, 'new') ; end
def onOpenModel(model) ; model_changing(model, 'open') ; end
def onQuit() ; quit_sketchup ; end

def onDeleteModel(model) ; on_close_model ; end
def onSaveModel(model) ; on_save_model ; end
	
def onActiveToolChanged(tools, tool_name, tool_id) ; notify_event(tool_name, tool_id) unless tool_id == 0; end
def onToolStateChanged(tools, tool_name, tool_id, tool_state) ; notify_event ; end

def onSelectionBulkChange(selection) ; notify_selection ; end
def onSelectionAdded(selection, elt) ; notify_selection ; end
def onSelectionRemoved(selection, elt) ; notify_selection ; end
def onSelectionCleared(selection) ; notify_selection ; end
def onViewChanged(view) ; notify_event ; end

def notify_selection 
	notify_event if Sketchup.active_model.tools.active_tool_id != 0 && Sketchup.active_model.tools.active_tool_name =~ /ctionTool/i
end

#----------------------------------------------------------------------------------------------
# Class methods and global parameters
#----------------------------------------------------------------------------------------------

@@attr_dico = "SUClock"
@@attr_path = "Path"
@@attr_rel_path = "RelPath"

#Check if the current model is tracked or trackable
def Tracker.get_model_prjpath
	model = Sketchup.active_model
	return nil unless model
	model.get_attribute @@attr_dico, @@attr_path
end

#Method called once when SU starts
def initialize
	@hsh_projects = {}	
	@previous_project = nil

	@main_cmd = MYPLUGIN.get_command :MCLK_LAUNCH_

	@tip_show_clock = T6[:TIP_ShowClock]	
	@tip_change_settings = T6[:TIP_ChangeSettings]	
	@tip_attach_model = T6[:TIP_AttachModel]	
	
	#Creating the clock structure
	@clock = VisualClock.new self

	#Initialize temporary variables
	Project.init_temporary_env
	
	#Add the Application Observer to track change of models
	Sketchup.add_observer self
	
	#Check if current model is tracked
	declare_model	
end

#-----------------------------------------------------------------------------------
# Events changing the model
#-----------------------------------------------------------------------------------

#Record an event to the model
def model_changing(model, event, must_exist=false)
	@previous_project.save if @previous_project
	prj = declare_model must_exist
	TopDialog.close_dialog
	model.select_tool nil
end

#Attach the tracking for a model to a project and start the model and tools observers
def attach_model(model, prj)
	model.add_observer self
	model.tools.add_observer self
	model.selection.add_observer self
	model.active_view.add_observer self
end

#Remove the tracking for a model
def detach_model(model)
	model.set_attribute @@attr_dico, @@attr_path, nil
	model.set_attribute @@attr_dico, @@attr_rel_path, nil
end

#Model is closed (Mac only)
def on_close_model
	prj = current_project
	@clock.update prj
end

#Model is closed (Mac only)
def on_save_model
	prj = current_project
	prj.save if prj
end

#Quit Sketchup - general cleanup
def quit_sketchup
	prj = @previous_project
	prj.save if prj
end

#-----------------------------------------------------------------------------------
# Managing current model
#-----------------------------------------------------------------------------------

#Get the current project and update the previous one
def current_project
	model = Sketchup.active_model
	return nil unless model
	prjpath = model.get_attribute @@attr_dico, @@attr_path
	
	@previous_project = @hsh_projects[prjpath]
end

#Attach a Project object to a model if the model is tracked
def declare_model(must_exist=false)
	model = Sketchup.active_model
	return unless model
	prjpath = model.get_attribute @@attr_dico, @@attr_path
	
	#Checking if the Clock file exist - If not ask for relocating the file
	prjpath = rescue_path prjpath, must_exist if prjpath
	unless prjpath && FileTest.exist?(prjpath)
		@clock.update nil
		update_main_tooltip nil
		attach_model model, nil 
		return nil
	end	
	
	#Create the Project structure if not already created
	@declaring_model = true
	key = prjpath
	prj = @hsh_projects[key]
	register_attachment prjpath

	unless prj
		prj = Project.new prjpath, @clock
		@hsh_projects[key] = prj
	end
	
	#Attaching the project to the model 
	@clock.update prj
	attach_model model, prj
	update_main_tooltip prj
	@declaring_model = false
	
	prj
end

#Check Existence of the Project file
def rescue_path(prjpath, ask=false)
	return prjpath if FileTest.exist?(prjpath)
	
	#Trying with the relative path
	model = Sketchup.active_model
	relpath = model.get_attribute @@attr_dico, @@attr_rel_path
	if relpath
		relpath = File.expand_path relpath, model.path
	end	
	
	#Asking for the file
	unless FileTest.exist?(relpath)
		return nil unless ask
		status = UI.messagebox "#{T6[:MSG_SearchAlternate1]}\n#{prjpath}\n\n#{T6[:MSG_SearchAlternate2]}", MB_YESNO
		return nil unless status == 6
		prjpath = UI.openpanel T6[:DLG_AskSelectFile], File.dirname(relpath), File.basename(relpath)
		return nil unless prjpath
		prjpath = prjpath.gsub(/\\/, '/')
	end	
	prjpath
end

#Notify the attachment of the current model to a project	
def register_attachment(prjpath=nil)	
	model = Sketchup.active_model
	return detach_model(model) unless prjpath
	
	relative_path = Traductor.relative_path prjpath, model.path
	
	oldp = model.get_attribute @@attr_dico, @@attr_path
	oldr = model.get_attribute @@attr_dico, @@attr_rel_path
	
	model.set_attribute @@attr_dico, @@attr_path, prjpath if prjpath != oldp
	model.set_attribute @@attr_dico, @@attr_rel_path, relative_path if relative_path != oldr
end

#Notify the detachment of the current model	
def notify_detachment(prj)	
	return unless prj
	prj.save
	detach_model Sketchup.active_model
	@clock.hide_clock
end

#Signal a change in model so that the main cmd tooltip is updated
def signal_change
	prj = current_project
	update_main_tooltip prj
end

#Central method to notify an Event
def notify_event(tool_name=nil, tool_id=nil)
	return if @declaring_model
	prj = current_project
	Project.update_last_tracked prj
	if prj
		prj.record_event tool_name, tool_id
	else
		@clock.hide_clock
	end	
end

#-----------------------------------------------------------------------------------
# GUI and User interactions
#-----------------------------------------------------------------------------------

#Compute and update the main command tooltip
def update_main_tooltip(prj=nil)
	return unless @main_cmd
	if prj
		tip = prj.cmd_tooltip
		if !@clock.visible?
			tip += ' - ' + @tip_show_clock
		else
			tip += " - " + @tip_change_settings
		end
	else
		tip = @tip_attach_model
	end
	@main_cmd.tooltip = tip
end

#Set or Unset Clock tracking for a model skp file
def toggle_clock
	#Getting the current project if any
	declare_model true
	prj = current_project
	
	#Model is not tracked
	return Project.dialog_configuration(self) unless prj
	
	#Model is already tracked
	if @clock.visible?
		Project.dialog_configuration self, prj
	else
		prj.hide_project false
		@clock.update prj
	end	
end
	
#Invoke the top dialog box for configuration of the project	
def config_display
	prj = current_project
	Project.dialog_configuration self, prj if prj
end
	
#Center the clock in the screen	
def clock_within_screen
	@clock.center_on_screen
end
		
end	#class Tracker

#=============================================================
#--------------------------------------------------------------------------------------------------------------------------
# Class VisualClock: Manage the display of the clock
#--------------------------------------------------------------------------------------------------------------------------
#=============================================================

class VisualClock

#Initialize the clock structure
def initialize(tracker)
	@tracker = tracker
	@wdlg = nil
	@displayed_time = nil
	@record = false

	#Sizes for Dialog box
	@sizex = 170
	@sizey = MYDEFPARAM[:DEFAULT_ClockHeight]
	@sizey = 50 unless @sizey

	#Parameters for HTML
	@id_time = "ID_TIME"
	@id_config = "ID_CONFIG"
	@id_play = "ID_PLAY"
	@color_gray = '#585858'	
	
	#Colors for alarm
	@bkcolor_ok = "black"
	@fkcolor_ok = "lightgreen"	
	@bkcolor_warning = "orange"
	@fkcolor_warning = "green"
	@bkcolor_over = "red"
	@fkcolor_over = "white"
	@bkcolor = @bkcolor_ok
	@fkcolor = @fkcolor_ok
	@status_alarm = 0
	
	@manual_position = RUN_ON_MAC
end

#Display the clock and center it on screendef center_on_screen
def center_on_screen
	return unless @proj
	update @proj
	view = Sketchup.active_model.active_view
	w = view.vpwidth
	h = view.vpheight
	x = (w - @sizex) / 2
	y = (h - @sizey) / 2
	@wdlg.set_position x, y
end

#Update the clock display (show, hide and update)
def update(proj=nil)
	return hide_clock() unless proj	
	
	#Reinitialize if change of project
	hide_clock() unless proj == @proj
	return if proj.hidden

	#Creating the dialog box (shadow box is used for giving back focus to Sketchup window)
	unless @wdlg
		@proj = proj
		check_alarm
		unless @wdlg		
			@wdlg = create_wdlg
			dlg_shadow = create_wdlg true
		end	
		transfer_html format_html
		@wdlg.set_html format_html
		if RUN_ON_MAC
			@wdlg.show_modal
			dlg_shadow.show_modal
		else
			@wdlg.show
			dlg_shadow.show
		end	
		dlg_shadow.close
		@tracker.signal_change
		
	#Just updating the display	
	else
		check_alarm
		stime = @proj.time_to_display
		if stime != @displayed_time
			@displayed_time = stime
			@wdlg.execute_script "update_time('#{stime}')"
			@tracker.signal_change
		end	
	end
end

#Transfer HTML to web dialog. On Mac, need to use set_file as of Safari 5.0.6	
def transfer_html(text_html)
	return unless @wdlg && text_html
	if RUN_ON_MAC
		delete_temp_file
		@tmpfile = File.join LibFredo6.tmpdir, "LibFredo_webdialog #{Time.now.to_f}.html"
		File.open(@tmpfile, "w") { |f| f.puts text_html }
		@wdlg.set_file @tmpfile if @wdlg
	else
		@wdlg.set_html text_html if @wdlg
	end	
end

def delete_temp_file
	File.unlink @tmpfile if @tmpfile
	@tmpfile = nil
end

#Close and delete the Web diag box
def hide_clock
	@proj = nil
	@wdlg.close if @wdlg
end

#Call back procedure when dialog box is closed
def on_close
	@proj.hide_project true if @proj
	@wdlg = nil
	@tracker.signal_change
	TopDialog.close_dialog
	delete_temp_file
end

#Create the web dialog box (main and shadow)
def create_wdlg(shadow=false)
	title  = @proj.prjname
	@wdlg_key = 'SUClock_Position'
	xpos = 500
	ypos = 400
		
	#Sizes for Dialog box
	@sizex = 170
	@sizey = MYDEFPARAM[:DEFAULT_ClockHeight]
	sy = @sizey + ((SU_MAJOR_VERSION == 7 && !RUN_ON_MAC) ? 5 : 0)

	if @manual_position
		regkey = ""
		ls = Traductor::Registry.wposition_load @wdlg_key
		xpos, ypos = ls if ls
	else
		regkey = @wdlg_key
	end
	wdlg = UI::WebDialog.new title, false, regkey, @sizex, sy, xpos, ypos, false
	unless shadow
		wdlg.add_action_callback("suclock_callback") { |dialog, params| callback(params) }
		wdlg.set_on_close { on_close }
	end	
	wdlg.set_position xpos, ypos if @manual_position
	wdlg.set_size @sizex, sy
	wdlg.set_background_color @bkcolor
	wdlg.navigation_buttons_enabled = false if SU_MAJOR_VERSION > 6
	wdlg
end

#Creating the HTML stream	
def format_html
	stime = @proj.time_to_display
	
	tip_time = @proj.displayed_tooltip
	@tip_config = T6[:TIP_CONFIG]
	@tip_record = T6[:TIP_RECORD]
	@tip_pause = T6[:TIP_PAUSE]

 	@img_config = Traductor::HTML.image_file Traductor::MYPLUGIN.picture_get("Button_Wrench.png")
 	@img_record = Traductor::HTML.image_file Traductor::MYPLUGIN.picture_get("Button_Play.png")
 	@img_pause = Traductor::HTML.image_file Traductor::MYPLUGIN.picture_get("Button_Pause.png")
	
	text = ""
	text += Traductor::T_HTML_DOC_TYPE
	text += Traductor::T_HTML_UTF_TYPE
	text += Traductor::T_HTML_SCRIPT_TYPE
	text += built_in_script_js
	
	text += "<BODY>"
	
	text += "<TABLE ID='MainTable' CELLPADDING='0' cellspacing='0' BGCOLOR='#{@bkcolor}' style='position:absolute; left:0px; top:0px'><tr>"
	
	color = (@proj.paused) ? @color_gray : @fkcolor
	style = "STYLE='font-size:24px ; font-style:bold ; color:#{color} ; text-align:center'"
	h_action = "onmouseover='action(\"hover\")'"
	text += "<td width='#{@sizex-50}px' style='text-align:center'>"
	text += "<b><tt><SPAN " + style + " ID='#{@id_time}' TITLE='#{tip_time}' #{h_action}>" + stime + "</SPAN></tt></b>"
 	text += "</td>"

 	action = "onclick='action(\"config\")'" + ' ' + h_action
	attr = "style='cursor:pointer' TITLE='#{@tip_config}' #{action}"
	text += "<td width='25px'>"
	text += "<a #{attr}><img ID='#{@id_config}' src='#{@img_config}' height='16' width='16' border='0' /></a>"
	text += "</td>"

	img_play = (@proj.paused) ? @img_record : @img_pause
	action = "onclick='action(\"play\")'" + ' ' + h_action
	attr = "style='cursor:pointer' TITLE='#{@tip_pause}' #{action}"
	text += "<td width='25px'>"
	text += "<a #{attr}><img ID='#{@id_play}' src='#{img_play}' height='16' width='16' border='0'/></a>"
	text += "</td>"
	
	text += "</tr></table>"
	text += "</BODY>"
	text += "</HTML>"
	
	text
end

#Section containing built-in scripts in the clock HTML page
def built_in_script_js()

	text = "<SCRIPT>"
	
	#Background and foreground color
	text += "$bkcolor = '#{@bkcolor}' ;"
	text += "$fkcolor = '#{@fkcolor}' ;"
	
	#Sketchup call back
	text += "function action(msg) {	window.location = 'skp:suclock_callback@' + msg ; }"

	#Toggle the play / pause button
	text += "function set_recording(play) { "
	text += "document.getElementById('MainTable').style.backgroundColor = $bkcolor ; "
	text += "if (play) { document.getElementById('#{@id_play}').src = '#{@img_record}' ; "
	text += "document.getElementById('#{@id_play}').title = '#{@tip_record}' ; "
	text += "document.getElementById('#{@id_time}').style.color = '#{@color_gray}' ; }"
	text += "else { document.getElementById('#{@id_play}').src = '#{@img_pause}' ; "
	text += "document.getElementById('#{@id_play}').title = '#{@tip_pause}' ; "
	text += "document.getElementById('#{@id_time}').style.color = $fkcolor ; } }"

	#Update time
	text += "function update_time(stime) { document.getElementById('#{@id_time}').innerHTML = stime } ; "

	#Update tooltip
	text += "function update_tip(stip) { document.getElementById('#{@id_time}').title = stip } ; "

	#Update clock colors 
	text += "function update_colors(bkcolor, fkcolor) {  "
	text += "$bkcolor = bkcolor ;"
	text += "$fkcolor = fkcolor ;"
	text += "document.getElementById('MainTable').style.backgroundColor = $bkcolor ; "
	text += "document.getElementById('#{@id_time}').style.color = $fkcolor ; "
	text += "}"


	#gathering window position
	if @manual_position
		if RUN_ON_MAC
			text += "function wposition() { txt = 'wposition;' + window.screenX + ';' + window.screenY" +
					" + ';' + window.outerWidth + ';' + window.outerHeight ; action(txt) ;}"
			text += "window.onblur = function() { wposition() ; } ;"
		else
			text += "function wposition() { txt = 'wposition;' + (window.screenLeft-3) + ';' + (window.screenTop-22)" +
					" + ';' + (document.documentElement.offsetWidth+6) + ';' + (document.documentElement.offsetHeight+25) ; action(txt) ;}"  
			text += "window.onunload = function() { wposition() ; } ;"
		end
	end
	
	text += "</SCRIPT>"
	text
end

#Web dialog callback method
def callback(param)
	proj = @tracker.current_project
	Project.update_last_tracked proj
	proj.record_event if proj
	
	case param
	when /play/
		@wdlg.execute_script "set_recording(#{proj.toggle_recording})"
	when /hover/
		@wdlg.execute_script "update_tip('#{proj.displayed_tooltip}')"
	when /config/
		Project.dialog_configuration @tracker, @proj
	when /wposition;/
		Traductor::Registry.wposition_store @wdlg_key, $'
	end	
end

#Check if the clock is visible
def visible?
	(@wdlg) ? @wdlg.visible? : false
end

#Check colors
def check_alarm
	return unless @proj
	status = @proj.status_alarm
	return if @status_alarm == status 
	case status
	when 1
		@bkcolor = @bkcolor_warning
		@fkcolor = @fkcolor_warning
	when 2
		@bkcolor = @bkcolor_over
		@fkcolor = @fkcolor_over
	else
		@bkcolor = @bkcolor_ok
		@fkcolor = @fkcolor_ok
	end	
	@status_alarm = status
	@wdlg.execute_script "update_colors('#{@bkcolor}', '#{@fkcolor}') ;" if @wdlg
end

end	#class VisualClock

#=============================================================
#--------------------------------------------------------------------------------------------------------------------------
# Class Project: used for managing Projects in the sens of time tracking
#--------------------------------------------------------------------------------------------------------------------------
#=============================================================

class Project

attr_reader :session_time, :hidden, :paused, :prjname, :prjpath, :total_time, :idle, 
            :creation_date, :time_store_stats, :total_time_stats, :target_time, :alarm_threshold

#Extensions and class variables
@@ext_suclock = ".suclock"
@@reg_suclock = Regexp.new("\\" + @@ext_suclock + "\\Z", Regexp::IGNORECASE)
@@reg_valid = Regexp.new("\\ASUClock v(\\d+)", Regexp::IGNORECASE)
@@encode_today = "%Y/%m/%d"

#-------------------------------------------------------------------------------
# Tempoorary File management
#-------------------------------------------------------------------------------

@@tmpdir_default = (RUN_ON_MAC) ? "/tmp" : ENV["TEMP"].gsub(/\\/, '/')
@@tmpdir = @@tmpdir_default
@@init_temp = nil
@@mru_file = nil

#initialize the temporary environment directory
def Project.get_tmpdir ; @@tmpdir ; end
def Project.init_temporary_env
	@@tmpdir = MYDEFPARAM[:DEFAULT_TmpDir]
	@@tmpdir = @@tmpdir_default unless @@tmpdir && FileTest.exist?(@@tmpdir)
	@@mru_file = File.join(@@tmpdir, "suclock_mru.tmp")
	@@last_tracked_file = File.join(@@tmpdir, "suclock_last_tracked.tmp")
	@@init_temp = true
end

#-------------------------------------------------------------------------------
# Last Tracked management
#-------------------------------------------------------------------------------

@@last_tracked_time = Time.now.to_f
@@last_tracked_delta = 0
@@last_tracked_project_prev = nil
@@last_tracked_project = nil
@@last_tracked_file = nil

#Update the last tracked time
def Project.update_last_tracked(proj=nil)
	now = Time.now.to_f
	Project.init_temporary_env unless @@init_temp
	
	#Read file on Windows
	begin
		lines = IO.readlines(@@last_tracked_file)
		lv = lines[0].split ';'
		@@last_tracked_time = lv[0].to_f
		@@last_tracked_project = (lv[1]) ? lv[1].strip : nil
	rescue
		@@last_tracked_time = now
		@@last_tracked_project = nil
	end		
	
	#Manage delta
	@@last_tracked_delta = now - @@last_tracked_time
	@@last_tracked_project_prev = @@last_tracked_project
	@@last_tracked_time = now
	@@last_tracked_project = (proj) ? proj.prjpath : nil
	
	#Write file on Windows
	File.open(@@last_tracked_file, 'w') { |f| f.puts "#{@@last_tracked_time} ; #{@@last_tracked_project}" }
	
	@@last_tracked_delta
end

#-------------------------------------------------------------------------------
# MRU list of project files
#-------------------------------------------------------------------------------

#Get the list of recent projects
def Project.mru_list(proj=nil)
	hsh = Project.mru_read
	lst = []
	hsh.each do |path, val| 
		next if proj && proj.prjpath == path
		lst.push [File.basename(path, @@ext_suclock), File.dirname(path), val, path] 
	end	
	lst.sort! { |a, b| a[2] <=> b[2] }
	lst.reverse
end

#Read the list of recent projects
def Project.mru_read
	Project.init_temporary_env unless @@init_temp
	return {} unless FileTest.exist?(@@mru_file)
	hsh = {}
	IO.foreach(@@mru_file) do |line|
		path, date = line.split ';'
		date = 0 unless date
		hsh[path] = date.to_f
	end	
	hsh
end

#Store a project path in the list of recent projects
def Project.mru_store(prjpath)
	hsh = Project.mru_read
	return if hsh[prjpath]
	hsh[prjpath] = Time.now.to_f
	File.open(@@mru_file, 'w') do |f| 
		hsh.each { |key, val| f.puts "#{key};#{val}" }	
	end	
end

#Purge the list of recent projects
def Project.mru_purge
	return unless @@mru_file
	File.open(@@mru_file, 'w') do |f| 
		f.puts "" 
	end	
end

#-------------------------------------------------------------------------------
# Dialogs for Project Configuration
#-------------------------------------------------------------------------------

#Invoke the top dialog box
def Project.dialog_configuration(tracker, prj=nil)
	prj.stats_save if prj
		
	TopDialog.display tracker, prj
end

#Ask the path to Project file	
def Project.ask_path(creation=false)
	model = Sketchup.active_model
	modelpath = model.path
	slash = (RUN_ON_MAC) ? '/' : "\\"
	
	#building the file and directory names
	if modelpath.empty?
		fdir = ""
		fname = "*" + @@ext_suclock
	else
		fdir = File.dirname(modelpath) + slash
		if creation
			fname = File.basename modelpath
			fname = fname.sub /\.skp/, @@ext_suclock
		else
			fname = "*" + @@ext_suclock
		end
	end
	
	#Asking for the file
	if creation
		path = UI.savepanel T6[:DLG_AskCreateFile], fdir, fname
	else	
		path = UI.openpanel T6[:DLG_AskSelectFile], fdir, fname
	end	
	return nil unless path
	
	#Cleaning the backslashes and adding the extension if not present
	path = path.gsub(/\\/, '/')
	
	#Checking this is a correct file
	if FileTest.exist?(path) && !Project.file_is_valid?(path)
		UI.messagebox T6[:MSG_InvalidFile, path]
		return nil
	end
	
	path += @@ext_suclock unless path =~ @@reg_suclock
	if FileTest.exist?(path) && !Project.file_is_valid?(path)
		UI.messagebox T6[:MSG_InvalidFile, path]
		return nil
	end

	#Creating the file
	unless FileTest.exist?(path)
		File.open(path, 'w') do |f| 
			def_idle = MYDEFPARAM[:DEFAULT_IdleTime] * 60
			f.puts "SUCLOCK v1 ; 0 ; #{def_idle} ; false ; #{Time.now.to_f}"
		end	
	end
	
	path
end

#-------------------------------------------------------------------------------
# Manipulation of Projects
#-------------------------------------------------------------------------------

#Check if a file is a valid SUClock file
def Project.file_is_valid?(path)
	return false unless path =~ /\.suclock\Z/i
	line = File.open(path, 'r').gets
	(line =~ @@reg_valid)
end

#Get the name of a project from its path
def Project.name_from_path(path)
	File.basename path, @@ext_suclock
end

def Project.get_total_time(prjpath)
	dirname = File.dirname prjpath
	tempclock = File.join @@tmpdir, "#{File.basename(dirname)}__#{File.basename prjpath}.txt"
	begin
		if FileTest.exist?(tempclock)
			line = File.open(tempclock, 'r').gets
		else
			line = File.open(prjpath, 'r').gets
		end	
		lv = line.strip.split ';'
		return lv[1].to_f
	rescue
		return 0
	end	
end

#Reset another Project
def Project.reset_other_project(prjpath)
	#Name of project files
	return false unless FileTest.exist?(prjpath)
	prjname = Project.name_from_path prjpath
	dirname = File.dirname prjpath
	tempstats = File.join @@tmpdir, "#{File.basename(dirname)}__#{prjname}.sustats.txt"
	tempclock = File.join @@tmpdir, "#{File.basename(dirname)}__#{File.basename prjpath}.txt"
	
	#Reading the parameters of the project file
	idle_time = MYDEFPARAM[:DEFAULT_IdleTime] * 60
	target_time = 0
	alarm_threshold = MYDEFPARAM[:DEFAULT_AlarmThreshold] / 100.0
	hidden = false
	line = File.open(prjpath, 'r').gets
	if (line =~ @@reg_valid)
		lv = line.strip.split ';'
		idle = lv[2].to_f if lv[2]
		hidden = ((lv[3] =~ /true/i) ? true : false)
		target_time = lv[6].to_i if lv[6]
		alarm_threshold = lv[7].to_f if lv[7]
	end
	
	#Resetting the project by updating all attached files
	Project.init_temporary_env unless @@last_tracked_file
	begin
		now = Time.now.to_f
		File.open(tempstats, 'w') { |f| f.puts "" } if FileTest.exist?(tempstats)
		File.open(prjpath, 'w') do |f| 
			f.puts "SUCLOCK v1 ; 0 ; #{idle} ; #{hidden} ; #{now} ; #{now} ; #{target_time} ; #{alarm_threshold}"
		end	
		if FileTest.exist?(tempclock)
			File.open(tempclock, 'w') do |f| 
				f.puts "SUCLOCK v1 ; 0 ; #{idle} ; #{hidden} ; #{now} ; #{now} ; #{target_time} ; #{alarm_threshold}"
			end	
		end	
		File.open(@@last_tracked_file, 'w') { |f| f.puts "#{now} ; #{prjpath}" } if FileTest.exist?(@@last_tracked_file)
	rescue
		return false
	end	
	true
end

#-------------------------------------------------------------------------------
# Project class instance methods
#-------------------------------------------------------------------------------

#Initialize the Project structure
def initialize(prjpath, clock)
	@clock = clock
	
	#Tooltips for the clock
	@tip_project = T6[:TIP_Project]
	@tip_total = T6[:TIP_Total]
	@tip_session = T6[:TIP_Session]
	
	#Default parameter
	@def_idle = MYDEFPARAM[:DEFAULT_IdleTime] * 60
	@def_alarm_threshold = MYDEFPARAM[:DEFAULT_AlarmThreshold] / 100.0

	#Default parameters (general)
	@interval_save = 300	
	@idle = @def_idle
	@target_time = 0
	@alarm_threshold = @def_alarm_threshold
	@paused = false
	@hidden = false

	#Initialization
	@creation_date = Time.now.to_f
	@session_time = 0
	@total_time = 0
	@time_displayed = Project.format_duration(@total_time)
	@initial_time = 0	
	@start_elapsed = @time_last_save = Time.now.to_f
	@last_tracked = @start_elapsed
	@last_read = 0
	@last_write = 0
	
	#Project path and name
	@prjpath = prjpath
	dirname = File.dirname @prjpath
	@basename = File.basename @prjpath
	@basename =~ @@reg_suclock
	@prjname = $`
	Project.mru_store @prjpath
	
	#Temporary tracking file
	@tempclock = File.join @@tmpdir, "#{File.basename(dirname)}__#{@basename}.txt"
	@tempstats = File.join @@tmpdir, "#{File.basename(dirname)}__#{@prjname}.sustats.txt"
	
	#Initializing stats variables
	stats_reset
	
	#New project
	auto_write unless FileTest.exist?(@prjpath)
	
	#Loading the parameters for the project
	auto_read
	@creation_date = Time.now.to_f unless @creation_date
	auto_write
	@idle = 300 if @idle < 60
	
	#Notification of activity
	Project.update_last_tracked self
	record_event
end

#Reset all data to zero for the project
def reset_to_zero
	@total_time = 0
	@total_time_stats = 0
	@session_time = 0
	now = Time.now.to_f
	@creation_date = now
	stats_reset
	auto_write
	File.open(@tempstats, 'w') { |f| f.puts "" }
	@time_store_stats = now 
	File.open(@prjpath, 'w') do |f| 
		f.puts "SUCLOCK v1 ; #{@total_time_stats} ; #{@idle} ; #{@hidden} ; #{@creation_date} ; #{now} ; #{@target_time} ; #{@alarm_threshold}"
	end	
	Project.init_temporary_env unless @@last_tracked_file
	File.open(@@last_tracked_file, 'w') { |f| f.puts "#{now} ; #{@prjpath}" }
	Project.update_last_tracked self
	record_event
end

#Modify the idle time duration
def change_idle_time(new_idle)
	@idle = new_idle
	auto_write
end
	
#Modify the target time duration
def change_target_time(new_target)
	@target_time = new_target
	@clock.check_alarm
	auto_write
	@clock.update self
end

#Modify the idle alarm threshold
def change_alarm_threshold(new_alarm)
	@alarm_threshold = new_alarm / 100.0
	@clock.check_alarm
	auto_write
	@clock.update self
end
	
#Check the status of alarm
def status_alarm
	return 0 if @target_time == 0
	return 2 if @total_time > @target_time
	return 1 if @total_time > @alarm_threshold * @target_time
	0
end
	
#Set the Clock display mode for the project
def hide_project(status)
	@hidden = (status) ? true : false
	auto_write
end
	
#Toggle the recording of time for the project	
def toggle_recording
	@paused = !@paused
end

#Record an Event indicating theat the project is active
def record_event(tool_name=nil, tool_id=nil)
	return if @paused
	now = Time.now.to_f
		
	#Read the total time
	auto_read
	
	#Tracking the time unless idle
	diff = @@last_tracked_delta
	if diff < @idle && @@last_tracked_project_prev == @prjpath
		@session_time += diff
		@total_time += diff
		stats_register_event diff
		auto_write
	end	
		
	#Saving the stats
	stats_save if (now - @time_last_save) > @interval_save
	
	#Change of tool
	if tool_id && tool_id != 0
		@last_tool_id = tool_id
		@last_tool_name = tool_name
	end	
	
	#Updating the clock
	@time_displayed = Project.format_duration(@total_time)
	TopDialog.update_time
	begin
		@clock.update self	
	rescue
	end
end

#Return the time to be shown in the clock
def time_to_display
	@time_displayed
end

#Utility to show the hours and minutes
def Project.format_duration(t)
	h = (t / 3600).floor
	m = ((t - h * 3600) / 60).floor
	"#{h}h " + sprintf('%02d', m) 
end

#Dynamic tooltip for the clock
def displayed_tooltip
	tip = "#{@tip_project} #{@prjname}"
	tip += "\\n#{@tip_total} #{Traductor.format_duration_hms(@total_time)}"
	tip += "\\n#{@tip_session} #{Traductor.format_duration_hms(@session_time)}"
	tip
end

#Dynamic tooltip for the Command button
def cmd_tooltip
	tip = "#{@tip_project} #{@prjname}"
	tip
end

#Save project information (generate an event first
def save
	@last_write = 0
	@last_read = 0
	record_event
	stats_save
	Project.mru_store @prjpath
end

#------------------------------------------------------------------------------------------
# Statistics Recording
#------------------------------------------------------------------------------------------

def stats_get_list(type)
	hsh = @hsh_full_stats[type]
	hsh
end

#Reset the stats variables
def stats_reset
	model = Sketchup.active_model
	return unless model
	@last_tool_id = model.tools.active_tool_id
	@last_tool_name = (@last_tool_id != 0) ? model.tools.active_tool_name : "SelectionTool"
	@hsh_tools = Hash.new 0
	@hsh_dates = Hash.new 0
	@hsh_files = Hash.new 0	
	
	@hsh_stats = { 'T' => @hsh_tools, 'D' => @hsh_dates, 'F' => @hsh_files }
	@time_last_save = Time.now.to_f
end

#Register the stats for the event
def stats_register_event(diff)
	today = Time.now.strftime @@encode_today
	current_path = Sketchup.active_model.path
	tool_name = Traductor::Ruby.clean_tool_name @last_tool_name, @last_tool_id
	
	File.open(@tempstats, "a") do |f|
		f.puts "T ; #{tool_name} ; #{diff}"
		f.puts "F ; #{current_path} ; #{diff}"
		f.puts "D ; #{today} ; #{diff}"
	end	
end

#Read the header part only of the suclock file
def stats_read_header
	result = [0, @def_idle, false, Time.now.to_f, 0, 0.75]
	if FileTest.exist?(@prjpath)
		line = File.open(@prjpath, 'r').gets
		if (line =~ @@reg_valid)
			lv = line.strip.split ';'
			result = [lv[1].to_f, lv[2].to_f, ((lv[3] =~ /true/i) ? true : false), lv[4].to_f]
			result.push((lv[6]) ? lv[6].to_i : 0) 
			result.push((lv[7]) ? lv[7].to_f : @def_alarm_threshold) 
		end	
	end
	result
end

#Read the tempory stats file
def stats_read_temp
	stats_reset
	return unless FileTest.exist?(@tempstats)
	lines = IO.readlines(@tempstats)
	lines.each do |line|
		line = line.strip
		next if line.empty?
		lv = line.strip.split ';'
		hsh = @hsh_stats[lv[0].strip]
		hsh[lv[1].strip] += lv[2].to_f
	end	
	
	#Clearing the file
	File.open(@tempstats, 'w') { |f| f.puts "" }
end

#Save and Merge statistics
def stats_save
	
	#Consolidating current stats from temporary file (Windows)
	stats_read_temp
	
	#Reading the file if it exists
	if FileTest.exist?(@prjpath)
		lines = IO.readlines(@prjpath)
		lines.each do |line|
			lv = line.strip.split ';'
			if lv[0] =~ /SUCLOCK/
				@total_time_stats = lv[1].to_f
				@time_store_stats = lv[5].to_f
				next
			end
			hsh = @hsh_stats[lv[0]]
			hsh[lv[1]] += lv[2].to_f if hsh && lv[2]
		end	
	end
	
	#Checking if stats total is OK with project total time
	@total_time_stats = 0 
	@hsh_files.each { |key, val| @total_time_stats += val }
	
	#Saving the file
	now = Time.now.to_f
	@time_store_stats = now unless @time_store_stats
	File.open(@prjpath, 'w') do |f| 
		f.puts "SUCLOCK v1 ; #{@total_time_stats} ; #{@idle} ; #{@hidden} ; #{@creation_date} ; #{now} ; #{@target_time} ; #{@alarm_threshold}"
		@hsh_stats.each do |letter, hsh|
			hsh.each { |key, t| f.puts "#{letter};#{key};#{t}" unless key == "nil" }
		end	
	end	

	#Saving the full data
	@hsh_full_stats = @hsh_stats
	
	#Resetting the stats
	@total_time = @total_time_stats
	auto_write
	
	stats_reset
end

#Import data from another project
def stats_import(otherpath)
	#Computing the file names for the other project
	other_prjname = Project.name_from_path otherpath
	other_dirname = File.dirname otherpath
	otempstats = File.join @@tmpdir, "#{File.basename(other_dirname)}__#{other_prjname}.sustats.txt"
	
	#Reading the temporary stats file for the other project
	lines_temp = IO.readlines(otempstats) if FileTest.exist?(otempstats)
	lines_main = IO.readlines(otherpath) if FileTest.exist?(otherpath)
	
	#Adding the lines to the current project
	File.open(@tempstats, "a") do |f|
		lines_temp.each { |line| f.puts line unless line.strip.empty? }
		lines_main.each { |line| f.puts line unless line.strip.empty? || line =~ /\ASUCLOCK/i}
	end	
	stats_save
	Project.init_temporary_env unless @@last_tracked_file
	File.open(@@last_tracked_file, 'w') { |f| f.puts "#{Time.now.to_f} ; #{@prjpath}" }
	Project.update_last_tracked self
	record_event
end

#------------------------------------------------------------------------------------------
# File Reading and Writing
#  - Auto: persistent file assigned to a project
#------------------------------------------------------------------------------------------

#Time tracking file Read
def auto_write
	begin
		File.open(@tempclock, 'w') do |f| 
			f.puts "SUCLOCK v1; #{@total_time} ; #{@idle} ; #{@hidden} ; #{@creation_date} ; #{@target_time} ; #{@alarm_threshold}"
		end	
	rescue
	end
end

#Time tracking file Write
def auto_read
	if !FileTest.exist?(@tempclock)
		@total_time, @idle, @hidden, @creation_date, ttarget, ttalarm = stats_read_header
		return
	end	
	lines = IO.readlines(@tempclock)
	line = (lines.length > 1) ? "SU;" + lines[1] : lines[0]
	lv = line.split ';'
	@total_time, @idle, @hidden, @creation_date = [lv[1].to_f, lv[2].to_f, ((lv[3] =~ /true/i) ? true : false), lv[4].to_f]
	@target_time = (lv[5]) ? lv[5].to_i : 0
	@alarm_threshold = (lv[6]) ? lv[6].to_f : @def_alarm_threshold 

end

end	#class Project
	
end	#End Module SUClock
